//---------------------------------------------------------------------------
#ifndef Moa_CommandH
#define Moa_CommandH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_command {
//---------------------------------------------------------------------------
class ECommandError : public Exception
{
public:
	__fastcall ECommandError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
enum TCommandResult
{
	crCancelled,
	crCompleted,
	crUndoable,
	crRedoable,
};
//---------------------------------------------------------------------------
typedef Set<TCommandResult, crCancelled, crRedoable> TCommandResults;
//---------------------------------------------------------------------------
class TCommand : public TObject
{
private:
	TCommandResults FResults;
protected:
public:
	__fastcall TCommand();
	virtual __fastcall ~TCommand();
	virtual void __fastcall Execute() = 0;
	virtual void __fastcall Undo() = 0;
	virtual void __fastcall Redo() = 0;
	__property TCommandResults Results = {read=FResults, write=FResults};
};
//---------------------------------------------------------------------------
class TInvoker : public TObject
{
private:
	TObjectList* FCommands;
	TObjectList* FUndoCommands;
	TObjectList* FRedoCommands;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
	virtual void __fastcall ExecuteDone();
	virtual void __fastcall UndoDone();
	virtual void __fastcall RedoDone();
	virtual TCommand* __fastcall GetNextCommand();
	virtual TCommand* __fastcall GetNextUndoCommand();
	virtual TCommand* __fastcall GetNextRedoCommand();
public:
	__fastcall TInvoker();
	virtual __fastcall ~TInvoker();
	virtual void __fastcall Execute();
	virtual bool __fastcall CanExecute();
	virtual void __fastcall Undo();
	virtual bool __fastcall CanUndo();
	virtual void __fastcall Redo();
	virtual bool __fastcall CanRedo();
	__property TObjectList* Commands = {read=FCommands};
	__property TObjectList* UndoCommands = {read=FUndoCommands};
	__property TObjectList* RedoCommands = {read=FRedoCommands};
	__property TCommand* NextCommand = {read=GetNextCommand};
	__property TCommand* NextUndoCommand = {read=GetNextUndoCommand};
	__property TCommand* NextRedoCommand = {read=GetNextRedoCommand};
};
//---------------------------------------------------------------------------
} // End namespace Moa_command
using namespace Moa_command;
//---------------------------------------------------------------------------
#endif

