//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_ListViewModel.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
#include <Math.hpp>
#include "Moa_AutoPointer.h"
//---------------------------------------------------------------------------
namespace Moa_listviewmodel {
//---------------------------------------------------------------------------
__fastcall TListViewModelEntry::TListViewModelEntry() :
	TObject(),
	FValue(NULL),
	FState(0)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListViewModelEntry::~TListViewModelEntry()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TListViewModelData::Initialize()
{
	try {
		FEntries = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModelData::Uninitialize()
{
	delete FEntries;
	FEntries = NULL;
}
//---------------------------------------------------------------------------
__fastcall TListViewModelData::TListViewModelData(TListViewModel* AOwner) :
	TObject(),
	FOwner(AOwner),
	FEntries(NULL),
	FMultiSelect(false),
	FFocusIndex(-1),
	FSelCount(0),
	FSelIndex(-1)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TListViewModelData::~TListViewModelData()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void* __fastcall TListViewModelItem::GetValue()
{
	return static_cast<TListViewModelEntry*>(Data->Entries->Items[Index])->Value;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModelItem::SetValue(void* AValue)
{
	static_cast<TListViewModelEntry*>(Data->Entries->Items[Index])->Value = AValue;
}
//---------------------------------------------------------------------------
bool __fastcall TListViewModelItem::GetFocused()
{
	return (State & LVIS_FOCUSED) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModelItem::SetFocused(bool AFocused)
{
	if(AFocused) {
		State = State | LVIS_FOCUSED;
	}
	else {
		State = State & ~LVIS_FOCUSED;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TListViewModelItem::GetSelected()
{
	return (State & LVIS_SELECTED) != 0;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModelItem::SetSelected(bool ASelected)
{
	if(ASelected) {
		State = State | LVIS_SELECTED;
	}
	else {
		State = State & ~LVIS_SELECTED;
	}
}
//---------------------------------------------------------------------------
UINT __fastcall TListViewModelItem::GetState()
{
	return static_cast<TListViewModelEntry*>(Data->Entries->Items[Index])->State;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModelItem::SetState(UINT AState)
{
	UINT LOldState = State;
	UINT LNewState = AState;
	if(LNewState == LOldState) {
		return;
	}
	static_cast<TListViewModelEntry*>(Data->Entries->Items[Index])->State = LNewState;
	if((LNewState & LVIS_FOCUSED) != 0) {
		if(Data->FocusIndex >= 0 && Data->FocusIndex != Index) {
			static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->FocusIndex])->State = static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->FocusIndex])->State & ~LVIS_FOCUSED;
		}
		Data->FocusIndex = Index;
	}
	else {
		if(Data->FocusIndex == Index) {
			Data->FocusIndex = -1;
		}
	}
	if((LNewState & LVIS_SELECTED) != 0) {
		if((LOldState & LVIS_SELECTED) == 0) {
			Data->SelCount = Data->SelCount + 1;
		}
		if(!Data->MultiSelect) {
			if(Data->SelIndex >= 0 && Data->SelIndex != Index) {
				static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->SelIndex])->State = static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->SelIndex])->State & ~LVIS_SELECTED;
			}
			Data->SelIndex = Index;
		}
	}
	else {
		if((LOldState & LVIS_SELECTED) != 0) {
			Data->SelCount = Data->SelCount - 1;
		}
		if(!Data->MultiSelect) {
			if(Data->SelIndex == Index) {
				Data->SelIndex = -1;
			}
		}
	}
}
//---------------------------------------------------------------------------
__fastcall TListViewModelItem::TListViewModelItem(TListViewModelData* AData) :
	TObject(),
	FData(AData),
	FIndex(-1)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TListViewModelItem::~TListViewModelItem()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
const UINT TListViewModel::StateMask = LVIS_CUT | LVIS_DROPHILITED | LVIS_FOCUSED | LVIS_SELECTED;
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Initialize()
{
	try {
		FData = new TListViewModelData(this);
		FItem = new TListViewModelItem(Data);
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Uninitialize()
{
	delete FItem;
	FItem = NULL;

	delete FData;
	FData = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::DoInserting(int AIndex)
{
	if(OnInserting != NULL) {
		OnInserting(this, AIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::DoInsert(int AIndex)
{
	if(OnInsert != NULL) {
		OnInsert(this, AIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::DoDeleting(int AIndex)
{
	if(OnDeleting != NULL) {
		OnDeleting(this, AIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::DoDelete(int AIndex)
{
	if(OnDelete != NULL) {
		OnDelete(this, AIndex);
	}
}
//---------------------------------------------------------------------------
TListViewModelItem* __fastcall TListViewModel::GetItem(int AIndex)
{
	FItem->Index = AIndex;
	return FItem;
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::GetCount()
{
	return Data->Entries->Count;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SetCount(int ACount)
{
	if(Count < ACount) {
		while(Count < ACount) {
			Add();
		}
	}
	else if(Count > ACount) {
		while(Count > ACount) {
			Delete(Count - 1);
		}
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::GetCapacity()
{
	return Data->Entries->Capacity;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SetCapacity(int ACapacity)
{
	Data->Entries->Capacity = ACapacity;
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::GetFocusIndex()
{
	return Data->FocusIndex;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SetFocusIndex(int AFocusIndex)
{
	if(AFocusIndex < 0) {
		if(FocusIndex >= 0) {
			Items[FocusIndex]->Focused = false;
		}
	}
	else {
		Items[AFocusIndex]->Focused = true;
	}
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::GetSelCount()
{
	return Data->SelCount;
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::GetSelIndex()
{
	return Data->SelIndex;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SetSelIndex(int ASelIndex)
{
	if(MultiSelect) {
		throw EStringListError("The feature is not available in the multi-select mode.");
	}
	if(ASelIndex < 0) {
		if(SelIndex >= 0) {
			Items[SelIndex]->Selected = false;
		}
	}
	else {
		Items[ASelIndex]->Selected = true;
	}
}
//---------------------------------------------------------------------------
bool __fastcall TListViewModel::GetMultiSelect()
{
	return Data->MultiSelect;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SetMultiSelect(bool AMultiSelect)
{
	if((AMultiSelect ? 1 : 0) == (MultiSelect ? 1 : 0)) {
		return;
	}
	Data->MultiSelect = AMultiSelect;
	ClearSelection();
}
//---------------------------------------------------------------------------
__fastcall TListViewModel::TListViewModel(TListView* AControl) :
	TObject(),
	FControl(AControl),
	FData(NULL),
	FItem(NULL),
	FOnInserting(NULL),
	FOnInsert(NULL),
	FOnDeleting(NULL),
	FOnDelete(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TListViewModel::~TListViewModel()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::Add()
{
	int LIndex = Count;
	Insert(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Insert(int AIndex)
{
	DoInserting(AIndex);
	{
		TAutoPointer<TListViewModelEntry> LItem = new TListViewModelEntry();
		Data->Entries->Insert(AIndex, LItem.Pointer);
		LItem.Release();
	}
	if(FocusIndex >= AIndex) {
		Data->FocusIndex = Data->FocusIndex + 1;
	}
	if(SelIndex >= AIndex) {
		Data->SelIndex = Data->SelIndex + 1;
	}
	DoInsert(AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::DeleteSelected()
{
	int LCount = Count;
	int LIndex = Count;
	for(int LI = 0; LI < LCount; ++LI) {
		if(SelCount == 0) {
			break;
		}
		if(Items[--LIndex]->Selected) {
			Delete(LIndex);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Delete(int AIndex)
{
	DoDeleting(AIndex);
	int LFocusIndex = FocusIndex;
	if(LFocusIndex == AIndex) {
		LFocusIndex = EnsureRange(FocusIndex, -1, Count - 1 - 1);
	}
	else if(FocusIndex > AIndex) {
		LFocusIndex = Data->FocusIndex - 1;
	}
	int LSelCount = SelCount;
	if(Items[AIndex]->Selected) {
		--LSelCount;
	}
	int LSelIndex = SelIndex;
	if(!MultiSelect) {
		if(LSelIndex == AIndex) {
			LSelIndex = -1;
		}
		else if(LSelIndex > AIndex) {
			LSelIndex = LSelIndex - 1;
		}
	}
	Data->Entries->Delete(AIndex);
	Data->FocusIndex = LFocusIndex;
	if(Data->FocusIndex >= 0) {
		static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->FocusIndex])->State = static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->FocusIndex])->State | LVIS_FOCUSED;
	}
	Data->SelCount = LSelCount;
	Data->SelIndex = LSelIndex;
	if(Data->SelIndex >= 0) {
		static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->SelIndex])->State = static_cast<TListViewModelEntry*>(Data->Entries->Items[Data->SelIndex])->State | LVIS_SELECTED;
	}
	DoDelete(AIndex);
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Clear()
{
	while(Count > 0) {
		Delete(Count - 1);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::SelectAll()
{
	for(int LI = 0; LI < Count; ++LI) {
		Items[LI]->Selected = true;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::ClearSelection()
{
	for(int LI = 0; LI < Count; ++LI) {
		Items[LI]->Selected = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::ReverseSelection()
{
	for(int LI = 0; LI < Count; ++LI) {
		Items[LI]->Selected = !Items[LI]->Selected;
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Sort(TListViewModelSortCompareMethod ACompareMethod)
{
	struct TLocal
	{
		static int __fastcall Compare(void* AItem1, void* AItem2)
		{
			TListViewModelSortItem* LItem1 = static_cast<TListViewModelSortItem*>(AItem1);
			TListViewModelSortItem* LItem2 = static_cast<TListViewModelSortItem*>(AItem2);
			int LResult = LItem1->CompareMethod(LItem1->Control, LItem1->Index, LItem2->Index);
			if(LResult == 0) {
				LResult = LItem1->Index - LItem2->Index;
			}
			return LResult;
		}
	};

	TListViewModelOrder LOrder;
	{
		DynamicArray<TListViewModelSortItem> LSortItems;
		LSortItems.Length = Count;
		for(int LI = 0; LI < Count; ++LI) {
			LSortItems[LI].Control = this;
			LSortItems[LI].Index = LI;
			LSortItems[LI].CompareMethod = ACompareMethod;
		}
		TAutoPointer<TList> LSortList = new TList();
		LSortList->Capacity = Count;
		for(int LI = 0; LI < Count; ++LI) {
			LSortList->Add(&LSortItems[LI]);
		}
		LSortList->Sort(TLocal::Compare);
		LOrder.Length = Count;
		for(int LI = 0; LI < Count; ++LI) {
			LOrder[LI].OldIndex = static_cast<TListViewModelSortItem*>(LSortList->Items[LI])->Index;
			LOrder[LI].NewIndex = LI;
		}
	}
	Reorder(LOrder);
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Invert()
{
	TListViewModelOrder LOrder;
	LOrder.Length = Count;
	for(int LI = 0; LI < Count; ++LI) {
		LOrder[LI].OldIndex = LI;
		LOrder[LI].NewIndex = Count - 1 - LI;
	}
	Reorder(LOrder);
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Exchange(int AIndex1, int AIndex2)
{
	if(AIndex1 == AIndex2) {
		return;
	}
	Data->Entries->Exchange(AIndex1, AIndex2);
	if(FocusIndex == AIndex1) {
		Data->FocusIndex = AIndex2;
	}
	else if(FocusIndex == AIndex2) {
		Data->FocusIndex = AIndex1;
	}
	if(!MultiSelect) {
		if(SelIndex == AIndex1) {
			Data->SelIndex = AIndex2;
		}
		else if(SelIndex == AIndex2) {
			Data->SelIndex = AIndex1;
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::MoveSelected(int ADirection)
{
	if(ADirection == 0) {
		return;
	}
	int LRest = SelCount;
	int LIncrement = ADirection < 0 ? 1 : -1;
	int LOldIndex = (ADirection < 0 ? 0 : (Count - 1)) - LIncrement;
	int LNewIndex = LOldIndex - LIncrement;
	for(int LI = 0; LI < Count; ++LI) {
		if(LRest == 0) {
			break;
		}
		LOldIndex += LIncrement;
		LNewIndex += LIncrement;
		if(!Items[LOldIndex]->Selected) {
			continue;
		}
		--LRest;
		if(LI == 0 || Items[LNewIndex]->Selected) {
			continue;
		}
		Exchange(LOldIndex, LNewIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Move(int AOldIndex, int ANewIndex)
{
	struct TLocal
	{
		static int __fastcall DetermineNewIndex(int AIndex, int AOldIndex, int ANewIndex)
		{
			int LMinIndex = (AOldIndex < ANewIndex) ? AOldIndex : ANewIndex;
			if(AIndex < LMinIndex) {
				return AIndex;
			}
			int LMaxIndex = (AOldIndex < ANewIndex) ? ANewIndex : AOldIndex;
			if(AIndex > LMaxIndex) {
				return AIndex;
			}
			if(AIndex == AOldIndex) {
				return ANewIndex;
			}
			return (AOldIndex < ANewIndex) ? (AIndex - 1) : (AIndex + 1);
		}
	};

	if(AOldIndex == ANewIndex) {
		return;
	}
	Data->Entries->Move(AOldIndex, ANewIndex);
	Data->FocusIndex = TLocal::DetermineNewIndex(FocusIndex, AOldIndex, ANewIndex);
	if(!MultiSelect) {
		Data->SelIndex = TLocal::DetermineNewIndex(SelIndex, AOldIndex, ANewIndex);
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Reorder(TListViewModelOrder AOrder)
{
	DynamicArray<TObject*> LEntries;
	LEntries.Length = AOrder.Length;
	for(int LI = 0; LI < AOrder.Length; ++LI) {
		LEntries[LI] = Data->Entries->Items[AOrder[LI].OldIndex];
	}
	bool LOwnsObjects = Data->Entries->OwnsObjects;
	Data->Entries->OwnsObjects = false;
	try {
		int LOldFocusIndex = FocusIndex;
		int LOldSelIndex = SelIndex;
		for(int LI = 0; LI < AOrder.Length; ++LI) {
			Data->Entries->Items[AOrder[LI].NewIndex] = LEntries[LI];
			if(LOldFocusIndex == AOrder[LI].OldIndex) {
				Data->FocusIndex = AOrder[LI].NewIndex;
			}
			if(LOldSelIndex == AOrder[LI].OldIndex) {
				Data->SelIndex = AOrder[LI].NewIndex;
			}
		}
	}
	catch(...) {
		Data->Entries->OwnsObjects = LOwnsObjects;
		throw;
	}
	Data->Entries->OwnsObjects = LOwnsObjects;
}
//---------------------------------------------------------------------------
int __fastcall TListViewModel::FindByValue(void* AValue)
{
	for(int LI = 0; LI < Count; ++LI) {
		if(Items[LI]->Value == AValue) {
			return LI;
		}
	}
	return -1;
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::Update(bool AUpdateState)
{
	Count = Control->Items->Count;
	MultiSelect = Control->MultiSelect;
	if(AUpdateState) {
		for(int LI = 0; LI < Count; ++LI) {
			Items[LI]->State = ListView_GetItemState(Control->Handle, LI, StateMask);
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TListViewModel::UpdateControl(bool AUpdateState)
{
	Control->Items->BeginUpdate();
	try {
		if(Control->Items->Count != Count) {
			Control->Items->Count = Count;
		}
		Control->MultiSelect = MultiSelect;
		if(AUpdateState) {
			for(int LI = 0; LI < Count; ++LI) {
				ListView_SetItemState(Control->Handle, LI, Items[LI]->State, StateMask);
			}
		}
	}
	catch(...) {
		Control->Items->EndUpdate();
		throw;
	}
	Control->Items->EndUpdate();
}
//---------------------------------------------------------------------------
} // End namespace Moa_listviewmodel
//---------------------------------------------------------------------------

