//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Task.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_task {
//---------------------------------------------------------------------------
void __fastcall TTaskThread::Execute()
{
	DoExecute();
}
//---------------------------------------------------------------------------
void __fastcall TTaskThread::DoExecute()
{
	if(OnExecute != NULL) {
		OnExecute(this);
	}
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::TTaskThread(bool ACreateSuspended) :
	TThread(ACreateSuspended),
	FOnExecute(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTaskThread::~TTaskThread()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTaskThread::CallSynchronize(TThreadMethod AMethod)
{
	Synchronize(AMethod);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TTask::SynchronizeInMainThread()
{
	SyncMethod(SyncData);
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoTerminateInMainThread(void* AData)
{
	if(OnTerminate != NULL) {
		OnTerminate(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoExceptionInMainThread(void* AData)
{
	if(OnException != NULL) {
		OnException(this, static_cast<Exception*>(AData));
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::ThreadExecute(TObject* ASender)
{
	try {
		try {
			Execute();
		}
		catch(Exception& LException) {
			DoException(&LException);
		}
		catch(...) {
			try {
				throw ETaskError("An unknown exception occurred.");
			}
			catch(Exception& LException) {
				DoException(&LException);
			}
		}
		DoTerminate();
		if(FreeOnTerminate) {
			delete this;
		}
	}
	catch(...) {
		// Do nothing
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoTerminate()
{
	if(OnTerminate != NULL) {
		Synchronize(&DoTerminateInMainThread, NULL);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoException(Exception* AException)
{
	if(OnException != NULL) {
		Synchronize(&DoExceptionInMainThread, AException);
	}
}
//---------------------------------------------------------------------------
__fastcall TTask::TTask() :
	TObject(),
	FThread(NULL),
	FSyncMethod(NULL),
	FSyncData(NULL),
	FCancelled(false),
	FFreeOnTerminate(false),
	FOnTerminate(NULL),
	FOnException(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::~TTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::Perform()
{
	if(Thread != NULL) {
		throw ETaskError("The task has been already started.");
	}
	try {
		FThread = new TTaskThread(true);
		Thread->FreeOnTerminate = true;
		Thread->OnExecute = ThreadExecute;
		Thread->Resume();
	}
	catch(...) {
		delete FThread;
		FThread = NULL;
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Cancel()
{
	FCancelled = true;
}
//---------------------------------------------------------------------------
void __fastcall TTask::Synchronize(TTaskMethod AMethod, void* AData)
{
	if(::GetCurrentThreadId() == System::MainThreadID) {
		AMethod(AData);
	}
	else {
		SyncMethod = AMethod;
		SyncData = AData;
		Thread->CallSynchronize(&SynchronizeInMainThread);
	}
}
//---------------------------------------------------------------------------
} // End namespace Moa_task
//---------------------------------------------------------------------------

