//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Rect.h"
#include "Moa_Window.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_window {
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Initialize()
{
	try {
		HWND LHandle = Classes::AllocateHWnd(HandleMessage);
		if(!::IsWindow(LHandle)) {
			throw EWindowError("The program could not create a window.");
		}
		FHandle = LHandle;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::Uninitialize()
{
	if(::IsWindow(Handle)) {
		Classes::DeallocateHWnd(Handle);
		FHandle = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMessageWindow::HandleMessage(TMessage& AMessage)
{
	if(DoMessage(AMessage)) {
		return;
	}
	AMessage.Result = ::DefWindowProc(Handle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
}
//---------------------------------------------------------------------------
bool __fastcall TMessageWindow::DoMessage(TMessage& AMessage)
{
	bool LHandled = false;
	if(OnMessage != NULL) {
		OnMessage(this, AMessage, LHandled);
	}
	return LHandled;
}
//---------------------------------------------------------------------------
__fastcall TMessageWindow::TMessageWindow() :
	TObject(),
	FHandle(NULL),
	FOnMessage(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMessageWindow::~TMessageWindow()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
HWND __fastcall TWindow::GetParent() const
{
	return ::GetParent(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TWindow::SetParent(HWND AParent)
{
	::SetLastError(NO_ERROR);
	::SetParent(Handle, AParent);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not set the parent window.");
	}
}
//---------------------------------------------------------------------------
HWND __fastcall TWindow::GetOwner() const
{
	return Windows[GW_OWNER];
}
//---------------------------------------------------------------------------
HWND __fastcall TWindow::GetWindow(UINT ARelationship) const
{
	HWND LWindow;
	::SetLastError(NO_ERROR);
	::GetWindow(Handle, ARelationship);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not get the handle of the window.");
	}
	return LWindow;
}
//---------------------------------------------------------------------------
bool __fastcall TWindow::GetEnabled() const
{
	return ::IsWindowEnabled(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TWindow::SetEnabled(bool AEnabled)
{
	::SetLastError(NO_ERROR);
	::EnableWindow(Handle, AEnabled);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not enable the window.");
	}
}
//---------------------------------------------------------------------------
bool __fastcall TWindow::GetVisible() const
{
	return ::IsWindowVisible(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TWindow::SetVisible(bool AVisible)
{
	if(AVisible) {
		LongAttrs[GWL_STYLE] = LongAttrs[GWL_STYLE] | WS_VISIBLE;
	}
	else {
		LongAttrs[GWL_STYLE] = LongAttrs[GWL_STYLE] & ~static_cast<LONG>(WS_VISIBLE);
	}
}
//---------------------------------------------------------------------------
LONG __fastcall TWindow::GetLongAttr(int AIndex) const
{
	::SetLastError(NO_ERROR);
	LONG LAttribute = ::GetWindowLong(Handle, AIndex);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not get the long-type attributes of the window.");
	}
	return LAttribute;
}
//---------------------------------------------------------------------------
void __fastcall TWindow::SetLongAttr(int AIndex, LONG ALongAttr)
{
	::SetLastError(NO_ERROR);
	::SetWindowLong(Handle, AIndex, ALongAttr);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not set the long-type attributes of the window.");
	}
}
//---------------------------------------------------------------------------
WORD __fastcall TWindow::GetWordAttr(int AIndex) const
{
	::SetLastError(NO_ERROR);
	LONG LAttribute = ::GetWindowWord(Handle, AIndex);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not get the word-type attributes of the window.");
	}
	return LAttribute;
}
//---------------------------------------------------------------------------
void __fastcall TWindow::SetWordAttr(int AIndex, WORD AWordAttr)
{
	::SetLastError(NO_ERROR);
	::SetWindowWord(Handle, AIndex, AWordAttr);
	if(::GetLastError() != NO_ERROR) {
		throw EWindowError("The program could not set the word-type attributes of the window.");
	}
}
//---------------------------------------------------------------------------
__fastcall TWindow::TWindow(HWND AHandle) :
	FHandle(AHandle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TWindow::TWindow(const TWindow& AOther) :
	FHandle(AOther.Handle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TWindow::~TWindow()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TWindow& __fastcall TWindow::operator =(const TWindow& AOther)
{
	if(&AOther == this) {
		return *this;
	}
	FHandle = AOther.FHandle;
	return *this;
}
//---------------------------------------------------------------------------
bool __fastcall TWindow::Exists() const
{
	return ::IsWindow(Handle);
}
//---------------------------------------------------------------------------
bool __fastcall TWindow::HasAncestor(HWND AHandle) const
{
	return ::IsChild(AHandle, Handle);
}
//---------------------------------------------------------------------------
bool __fastcall TWindow::HasDescendant(HWND AHandle) const
{
	return ::IsChild(Handle, AHandle);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TPoint __fastcall TTopLevelWindow::GetMinPos() const
{
	return TPoint(Placement.ptMinPosition);
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetMinPos(TPoint AMinPos)
{
	WINDOWPLACEMENT LPlacement = Placement;
	LPlacement.ptMinPosition = TPointHelper(AMinPos).AsWinPoint;
	Placement = LPlacement;
}
//---------------------------------------------------------------------------
TPoint __fastcall TTopLevelWindow::GetMaxPos() const
{
	return TPoint(Placement.ptMaxPosition);
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetMaxPos(TPoint AMaxPos)
{
	WINDOWPLACEMENT LPlacement = Placement;
	LPlacement.ptMaxPosition = TPointHelper(AMaxPos).AsWinPoint;
	Placement = LPlacement;
}
//---------------------------------------------------------------------------
TRect __fastcall TTopLevelWindow::GetNormalBounds() const
{
	return TRect(Placement.rcNormalPosition);
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetNormalBounds(TRect ANormalBounds)
{
	WINDOWPLACEMENT LPlacement = Placement;
	LPlacement.rcNormalPosition = TRectHelper(ANormalBounds).AsWinRect;
	Placement = LPlacement;
}
//---------------------------------------------------------------------------
TRect __fastcall TTopLevelWindow::GetBounds() const
{
	RECT LBounds = {0};
	if(!::GetWindowRect(Handle, &LBounds)) {
		throw EWindowError("The program could not get the bounds of the window.");
	}
	return LBounds;
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetBounds(TRect ABounds)
{
	if(!::MoveWindow(
		Handle,
		ABounds.left,
		ABounds.top,
		ABounds.Width(),
		ABounds.Height(),
		true)) {
		throw EWindowError("The program could not set the bounds of the window.");
	}
}
//---------------------------------------------------------------------------
UINT __fastcall TTopLevelWindow::GetState() const
{
	return Placement.showCmd;
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetState(UINT AState)
{
	WINDOWPLACEMENT LPlacement = Placement;
	LPlacement.showCmd = AState;
	Placement = LPlacement;
}
//---------------------------------------------------------------------------
WINDOWPLACEMENT __fastcall TTopLevelWindow::GetPlacement() const
{
	WINDOWPLACEMENT LPlacement = {0};
	LPlacement.length = sizeof(WINDOWPLACEMENT);
	if(!::GetWindowPlacement(Handle, &LPlacement)) {
		throw EWindowError("The program could not get the placement of the window.");
	}
	return LPlacement;
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindow::SetPlacement(const WINDOWPLACEMENT& APlacement)
{
	if(!::SetWindowPlacement(Handle, &APlacement)) {
		throw EWindowError("The program could not set the placement of the window.");
	}
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindow::TTopLevelWindow(HWND AHandle) :
	TWindow(AHandle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindow::TTopLevelWindow(const TTopLevelWindow& AOther) :
	TWindow(AOther.Handle)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindow::~TTopLevelWindow()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TTopLevelWindow& __fastcall TTopLevelWindow::operator =(const TTopLevelWindow& AOther)
{
	if(&AOther == this) {
		return *this;
	}
	*this = AOther;
	return *this;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
HWND __fastcall TTopLevelWindows::GetForegroundWindow() const
{
	return ::GetForegroundWindow();
}
//---------------------------------------------------------------------------
void __fastcall TTopLevelWindows::SetForegroundWindow(HWND AForegroundWindow)
{
	if(!::SetForegroundWindow(AForegroundWindow)) {
		throw EWindowError("The program could not set the foreground window.");
	}
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindows::TTopLevelWindows()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindows::TTopLevelWindows(const TTopLevelWindows& AOther)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTopLevelWindows::~TTopLevelWindows()
{
	// Do nothing
}
//---------------------------------------------------------------------------
TTopLevelWindows& __fastcall TTopLevelWindows::operator =(const TTopLevelWindows& AOther)
{
	return *this;
}
//---------------------------------------------------------------------------
} // namespace Moa_window
//---------------------------------------------------------------------------

