//---------------------------------------------------------------------------
// Version: 2.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include "Moa_AutoPointer.h"
#include "Moa_File.h"
#include "Moa_List.h"
#include "Oren_EditHandler.h"
#include "Oren_ListEdit.h"
#include "Oren_ListEditCommand.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Oren_edithandler {
//---------------------------------------------------------------------------
const AnsiString TEditHandler::iniAddRegularFilesDialog = "AddRegularFilesDialog";
const AnsiString TEditHandler::iniDereferenceShortcuts = "DereferenceShortcuts";
const AnsiString TEditHandler::iniFilterIndex = "FilterIndex";
//---------------------------------------------------------------------------
bool __fastcall TEditHandler::CanHandle(TCommand* ACommand)
{
	return ACommand->InheritsFrom(__classid(TEditCommand));
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Initialize()
{
	try {
		FAddRegularFilesDialog = new TOpenDialog(NULL);
		AddRegularFilesDialog->Title = "Add";
		AddRegularFilesDialog->Filter = "All|*";
		AddRegularFilesDialog->FilterIndex = 1;
		AddRegularFilesDialog->Options = AddRegularFilesDialog->Options << ofAllowMultiSelect << ofPathMustExist << ofFileMustExist;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Uninitialize()
{
	delete FAddRegularFilesDialog;
	FAddRegularFilesDialog = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::ChooseAddRegularFiles(TStrings* AFiles)
{
	AddRegularFilesDialog->FileName = AnsiString();
	if(!AddRegularFilesDialog->Execute()) {
		Abort();
	}
	AFiles->Assign(AddRegularFilesDialog->Files);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TEditHandler::ChooseAddDirectory()
{
	AnsiString LDirectory = GetCurrentDir();
	if(!SelectDirectory("Specify the directory.", WideString(), LDirectory)) {
		Abort();
	}
	TFiles().CurrentDirectory = LDirectory;
	return LDirectory;
}
//---------------------------------------------------------------------------
bool __fastcall TEditHandler::GetDereferenceShortcuts()
{
	return !AddRegularFilesDialog->Options.Contains(ofNoDereferenceLinks);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::SetDereferenceShortcuts(bool ADereferenceShortcuts)
{
	if((ADereferenceShortcuts ? 1 : 0) == (DereferenceShortcuts ? 1 : 0)) {
		return;
	}
	if(ADereferenceShortcuts) {
		AddRegularFilesDialog->Options = AddRegularFilesDialog->Options >> ofNoDereferenceLinks;
	}
	else {
		AddRegularFilesDialog->Options = AddRegularFilesDialog->Options << ofNoDereferenceLinks;
	}
}
//---------------------------------------------------------------------------
__fastcall TEditHandler::TEditHandler(TListViewModel* AControl, TInvoker* AInvoker, TObjectList* AItems) :
	THandler(AControl, AInvoker, AItems),
	FAddRegularFilesDialog(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TEditHandler::~TEditHandler()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Undo()
{
	if(!CanUndo()) {
		throw EHandlerError("The program cannot undo anything.");
	}
	Invoker->Undo();
}
//---------------------------------------------------------------------------
bool __fastcall TEditHandler::CanUndo()
{
	return Invoker->CanUndo() && CanHandle(Invoker->NextUndoCommand);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Redo()
{
	if(!CanRedo()) {
		throw EHandlerError("The program cannot redo anything.");
	}
	Invoker->Redo();
}
//---------------------------------------------------------------------------
bool __fastcall TEditHandler::CanRedo()
{
	return Invoker->CanRedo() && CanHandle(Invoker->NextRedoCommand);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Cut()
{
	Copy();
	Delete();
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Copy()
{
	Control->Update(true);
	TAutoPointer<TStringList> LFiles = new TStringList();
	LFiles->Capacity = Control->SelCount;
	for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
		if(!Control->Items[LI]->Selected) {
			continue;
		}
		--LRest;
		LFiles->Add(static_cast<TRenameItem*>(Control->Items[LI]->Value)->OldPath);
	}
	Clipboard()->AsText = LFiles->Text;
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Paste()
{
	TAutoPointer<TStrings> LFiles = new TStringList();
	LFiles->Text = Clipboard()->AsText;
	Add(LFiles);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::AddDirectory()
{
	TAutoPointer<TStringList> LFiles = new TStringList();
	LFiles->Add(ChooseAddDirectory());
	Add(LFiles);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::AddRegularFiles()
{
	TAutoPointer<TStringList> LFiles = new TStringList();
	ChooseAddRegularFiles(LFiles);
	Add(LFiles);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Add(TStrings* AFiles)
{
	TAutoPointer<TListEditCommand> LCommand = new TListEditCommand(Control);
	try {
		LCommand->Invoker->Commands->Capacity = AFiles->Count;
		TAutoPointer<TStringList> LHistory = new TPathList();
		LHistory->Capacity = Control->Count + AFiles->Count;
		LHistory->Duplicates = dupIgnore;
		LHistory->Sorted = true;
		for(int LI = 0; LI < Control->Count; ++LI) {
			LHistory->Add(static_cast<TRenameItem*>(Control->Items[LI]->Value)->OldPath);
		}
		for(int LI = 0; LI < AFiles->Count; ++LI) {
			AnsiString LFile = TFile(TPath(AFiles->Strings[LI]).Expand()).LongPath;
			if(!TStringListHelper(LHistory).TryAdd(LFile)) {
				continue;
			}
			TAutoPointer<TRenameItem> LItem = new TRenameItem(LFile);
			LCommand->Invoker->Commands->Add(new TAddSubCommand(Control, LItem.Pointer));
			Items->Add(LItem.Release());
		}
		if(LCommand->Invoker->Commands->Count == 0) {
			throw EHandlerError("No files to add to the list.");
		}
	}
	catch(EFileError& LException) {
		throw EHandlerError("The program can not add files to the list.");
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::Delete()
{
	TAutoPointer<TListEditCommand> LCommand = new TListEditCommand(Control);
	Control->Update(true);
	LCommand->Invoker->Commands->Capacity = Control->SelCount;
	for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
		int LIndex = Control->Count - 1 - LI;
		if(!Control->Items[LIndex]->Selected) {
			continue;
		}
		--LRest;
		LCommand->Invoker->Commands->Add(new TDeleteSubCommand(Control, LIndex));
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::SelectAll()
{
	Control->Update(true);
	Control->SelectAll();
	Control->UpdateControl(true);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::SelectByType(Oren_rename::TFileType AType)
{
	Control->Update(true);
	for(int LI = 0; LI < Control->Count; ++LI) {
		Control->Items[LI]->Selected = static_cast<TRenameItem*>(Control->Items[LI]->Value)->Type == AType;
	}
	Control->UpdateControl(true);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::SelectByResult(TRenameResult AResult)
{
	Control->Update(true);
	for(int LI = 0; LI < Control->Count; ++LI) {
		Control->Items[LI]->Selected = static_cast<TRenameItem*>(Control->Items[LI]->Value)->Result == AResult;
	}
	Control->UpdateControl(true);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::ReverseSelection()
{
	Control->Update(true);
	Control->ReverseSelection();
	Control->UpdateControl(true);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::LoadSettings(TMemIniFile* AFile, TIniSection* ASection)
{
	ASection->Enter(iniAddRegularFilesDialog);
	AddRegularFilesDialog->FilterIndex = AFile->ReadInteger(
		ASection->Section,
		iniFilterIndex,
		AddRegularFilesDialog->FilterIndex);
	ASection->Leave();

	DereferenceShortcuts = AFile->ReadBool(
		ASection->Section,
		iniDereferenceShortcuts,
		DereferenceShortcuts);
}
//---------------------------------------------------------------------------
void __fastcall TEditHandler::SaveSettings(TMemIniFile* AFile, TIniSection* ASection)
{
	ASection->Enter(iniAddRegularFilesDialog);
	AFile->WriteInteger(
		ASection->Section,
		iniFilterIndex,
		AddRegularFilesDialog->FilterIndex);
	ASection->Leave();

	AFile->WriteBool(
		ASection->Section,
		iniDereferenceShortcuts,
		DereferenceShortcuts);
}
//---------------------------------------------------------------------------
} // End namespace Oren_edithandler
//---------------------------------------------------------------------------

