//---------------------------------------------------------------------------
// Version: 2.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include <StrUtils.hpp>
#include "Moa_AutoPointer.h"
#include "Moa_File.h"
#include "Moa_List.h"
#include "Moa_Net.h"
#include "Moa_Text.h"
#include "Oren_NameEdit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Oren_nameedit {
//---------------------------------------------------------------------------
AnsiString __fastcall TRenameName::GetPart(TNamePart AKind)
{
	switch(AKind) {
	case npName:
		return Name;
	case npBaseName:
		return TPath(Name).BaseName;
	case npExtension:
		return TPath(Name).Extension;
	default:
		throw ENameEditError("The kind is not supported.");
	}
}
//---------------------------------------------------------------------------
void __fastcall TRenameName::SetPart(TNamePart AKind, const AnsiString& APart)
{
	TPath LPath = Name;
	switch(AKind) {
	case npName:
		LPath.Path = APart;
		break;
	case npBaseName:
		LPath.BaseName = APart;
		break;
	case npExtension:
		LPath.Extension = APart;
		break;
	default:
		throw ENameEditError("The kind is not supported.");
	}
	Name = LPath;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRenameName::ToLowerCase(const AnsiString& AName)
{
	return TTextCapitalizer().MakeLowerCase(AName);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRenameName::ToUpperCase(const AnsiString& AName)
{
	return TTextCapitalizer().MakeUpperCase(AName);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TRenameName::ToBookTitle(const AnsiString& AName)
{
	return TTextCapitalizer().MakeBookTitle(AName);
}
//---------------------------------------------------------------------------
__fastcall TRenameName::TRenameName() :
	FName()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TRenameName::TRenameName(const AnsiString& AName) :
	FName(AName)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TRenameName::TRenameName(const TRenameName& AOther) :
	FName(AOther.Name)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TRenameName::~TRenameName()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TRenameName::operator AnsiString()
{
	return Name;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::operator  =(const TRenameName& AOther)
{
	if(&AOther == this) {
		return *this;
	}
	FName = AOther.FName;
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::Insert(TInsertPosition APosition, const AnsiString& AText)
{
	switch(APosition) {
	case ipHead:
		Name = AText + Name;
		break;
	case ipTail:
		Name = Name + AText;
		break;
	default:
		throw ENameEditError("The position is not supported.");
	}
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::Overwrite(const AnsiString& AText)
{
	Name = AText;
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::Replace(const AnsiString& AOldText, const AnsiString& ANewText, bool AMatchCase)
{
	if(AMatchCase) {
		Name = AnsiReplaceStr(Name, AOldText, ANewText);
	}
	else {
		Name = AnsiReplaceText(Name, AOldText, ANewText);
	}
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::Capitalize(TCharCase ACase)
{
	switch(ACase) {
	case ccLowerCase:
		Name = ToLowerCase(Name);
		break;
	case ccUpperCase:
		Name = ToUpperCase(Name);
		break;
	case ccBookTitle:
		Name = ToBookTitle(Name);
		break;
	default:
		throw ENameEditError("The character case is not supported.");
	}
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::CharEncode(TCharEncoding AEncoding)
{
	switch(AEncoding) {
	case ceUtf8:
		Name = AnsiToUtf8(Name);
		break;
	case ceAnsi:
		Name = Name;
		break;
	default:
		throw ENameEditError("The character encoding is not supported.");
	}
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::CharDecode(TCharEncoding AEncoding)
{
	switch(AEncoding) {
	case ceUtf8:
		Name = Utf8ToAnsi(Name);
		break;
	case ceAnsi:
		Name = Name;
		break;
	default:
		throw ENameEditError("The character encoding is not supported.");
	}
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::UrlEncode()
{
	Name = TUrlCoder().Encode(Name);
	return *this;
}
//---------------------------------------------------------------------------
TRenameName& __fastcall TRenameName::UrlDecode()
{
	Name = TUrlCoder().Decode(Name);
	return *this;
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TCustomSrcLinesSpecs::Produce(int ACount, TStrings* ALines)
{
	TAutoPointer<TStrings> LLines = new TStringList();
	LLines->Capacity = ACount;
	for(int LI = 0; LI < ACount; ++LI) {
		LLines->Add(Text);
	}
	ALines->Assign(LLines);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TClipboardSrcLinesSpecs::Produce(int ACount, TStrings* ALines)
{
	TAutoPointer<TStrings> LLines = new TStringList();
	LLines->Text = Clipboard()->AsText;
	if(LLines->Count != ACount) {
		throw ENameEditError("The number of lines does not match.");
	}
	ALines->Assign(LLines);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TNumberSrcLinesSpecs::Produce(int ACount, TStrings* ALines)
{
	TAutoPointer<TStrings> LLines = new TStringList();
	LLines->Capacity = ACount;
	__int64 LIndex = Initial - Increment;
	for(int LI = 0; LI < ACount; ++LI) {
		LIndex += Increment;
		if(LIndex < 0) {
			throw ENameEditError("Negative numbers are not supported.");
		}
		AnsiString LText = LIndex;
		if(LText.Length() < Digits) {
			LText = AnsiString::StringOfChar(Padding, Digits - LText.Length()) + LText;
		}
		LLines->Add(Prefix + LText + Suffix);
	}
	ALines->Assign(LLines);
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
TSrcLinesSpecs& __fastcall TTextSrcSpecs::GetSelected()
{
	switch(Kind) {
	case slCustom:
		return Set.Custom;
	case slNumber:
		return Set.Number;
	case slClipboard:
		return Set.Clipboard;
	default:
		throw ENameEditError("The kind is not supported.");
	}
}
//---------------------------------------------------------------------------
} // End namespace Oren_nameedit
//---------------------------------------------------------------------------

