//---------------------------------------------------------------------------
#ifndef Oren_NameEditH
#define Oren_NameEditH
//---------------------------------------------------------------------------
// Version: 2.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Oren_nameedit {
//---------------------------------------------------------------------------
class ENameEditError : public Exception
{
public:
	__fastcall ENameEditError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
enum TNamePart
{
	npName,
	npBaseName,
	npExtension,
};
//---------------------------------------------------------------------------
enum TInsertPosition
{
	ipHead,
	ipTail,
};
//---------------------------------------------------------------------------
enum TCharCase
{
	ccLowerCase,
	ccUpperCase,
	ccBookTitle,
};
//---------------------------------------------------------------------------
enum TCharEncoding
{
	ceUtf8,
	ceAnsi,
};
//---------------------------------------------------------------------------
class TRenameName
{
private:
	AnsiString FName;
	AnsiString __fastcall GetPart(TNamePart AKind);
	void __fastcall SetPart(TNamePart AKind, const AnsiString& APart);
protected:
	static AnsiString __fastcall ToLowerCase(const AnsiString& AName);
	static AnsiString __fastcall ToUpperCase(const AnsiString& AName);
	static AnsiString __fastcall ToBookTitle(const AnsiString& AName);
public:
	__fastcall TRenameName();
	__fastcall TRenameName(const AnsiString& AName);
	__fastcall TRenameName(const TRenameName& AOther);
	virtual __fastcall ~TRenameName();
	virtual __fastcall operator AnsiString();
	virtual TRenameName& __fastcall operator  =(const TRenameName& AOther);
	virtual TRenameName& __fastcall Insert(TInsertPosition APosition, const AnsiString& AText);
	virtual TRenameName& __fastcall Overwrite(const AnsiString& AText);
	virtual TRenameName& __fastcall Replace(const AnsiString& AOldText, const AnsiString& ANewText, bool AMatchCase);
	virtual TRenameName& __fastcall Capitalize(TCharCase ACase);
	virtual TRenameName& __fastcall CharEncode(TCharEncoding AEncoding);
	virtual TRenameName& __fastcall CharDecode(TCharEncoding AEncoding);
	virtual TRenameName& __fastcall UrlEncode();
	virtual TRenameName& __fastcall UrlDecode();
	__property AnsiString Name = {read=FName, write=FName};
	__property AnsiString Parts[TNamePart AKind] = {read=GetPart, write=SetPart};
};
//---------------------------------------------------------------------------
struct TSrcLinesSpecs
{
	virtual void __fastcall Produce(int ACount, TStrings* ALines) = 0;
};
//---------------------------------------------------------------------------
struct TCustomSrcLinesSpecs : public TSrcLinesSpecs
{
	AnsiString Text;
	virtual void __fastcall Produce(int ACount, TStrings* ALines);
};
//---------------------------------------------------------------------------
struct TClipboardSrcLinesSpecs : public TSrcLinesSpecs
{
	virtual void __fastcall Produce(int ACount, TStrings* ALines);
};
//---------------------------------------------------------------------------
struct TNumberSrcLinesSpecs : public TSrcLinesSpecs
{
	AnsiString Prefix;
	AnsiString Suffix;
	int Initial;
	int Increment;
	int Digits;
	char Padding;
	virtual void __fastcall Produce(int ACount, TStrings* ALines);
};
//---------------------------------------------------------------------------
enum TSrcLinesSpecsKind
{
	slCustom,
	slNumber,
	slClipboard,
};
//---------------------------------------------------------------------------
struct TSrcLinesSpecsSet
{
	TCustomSrcLinesSpecs Custom;
	TClipboardSrcLinesSpecs Clipboard;
	TNumberSrcLinesSpecs Number;
};
//---------------------------------------------------------------------------
struct TTextSrcSpecs
{
	TSrcLinesSpecsKind Kind;
	TSrcLinesSpecsSet Set;
	TSrcLinesSpecs& __fastcall GetSelected();
};
//---------------------------------------------------------------------------
} // End namespace Oren_nameedit
using namespace Oren_nameedit;
//---------------------------------------------------------------------------
#endif

