//---------------------------------------------------------------------------
// Version: 2.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointer.h"
#include "Moa_File.h"
#include "Oren_ProgressDialog.h"
#include "Oren_RenameCommand.h"
#include "Oren_RenameTask.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Oren_renamecommand {
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Initialize()
{
	try {
		FInvoker = new TInvoker();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Uninitialize()
{
	delete FAsyncException;
	FAsyncException = NULL;

	delete FInvoker;
	FInvoker = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Execute(TRenameTask* ATask)
{
	FTask = ATask;
	Task->FreeOnTerminate = true;
	Task->OnProgress = TaskProgress;
	Task->OnTerminate = TaskTerminate;
	Task->OnException = TaskException;
	if(OrenProgressDialog == NULL) {
		OrenProgressDialog = new TOrenProgressDialog(Application);
	}
	OrenProgressDialog->Caption = "Rename";
	OrenProgressDialog->Message = "Renaming...";
	OrenProgressDialog->Command = "Cancel";
	OrenProgressDialog->CommandButton->Enabled = true;
	OrenProgressDialog->Progress = 0;
	OrenProgressDialog->OnExecute = ProgressDialogExecute;
	OrenProgressDialog->OnCloseQuery = ProgressDialogCloseQuery;
	Control->Control->Invalidate();
	OrenProgressDialog->Execute(true);
	Control->Control->Invalidate();
	Results = Results << crUndoable << crRedoable;
	if(OrenProgressDialog->Progress == 100) {
		Results = Results << crCompleted;
	}
	HandleAsyncException();
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::HandleAsyncException()
{
	if(AsyncException == NULL) {
		return;
	}
	try {
		if(AsyncException->InheritsFrom(__classid(EAbort))) {
			throw EAbort(AsyncException->Message);
		}
		else {
			throw ERenameError(AsyncException->Message);
		}
	}
	__finally {
		delete FAsyncException;
		FAsyncException = NULL;
	}
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::ProgressDialogExecute(TObject* ASender)
{
	Task->Perform();
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::ProgressDialogCloseQuery(TObject* ASender, bool& ACanClose)
{
	ACanClose = false;
	if(OrenProgressDialog->CommandButton->Enabled) {
		OrenProgressDialog->CommandButton->Enabled = false;
		Task->Cancel();
	}
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::TaskProgress(TObject* ASender, int AProgress)
{
	OrenProgressDialog->Progress = AProgress;
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::TaskTerminate(TObject* ASender)
{
	FTask = NULL;
	OrenProgressDialog->OnCloseQuery = NULL;
	OrenProgressDialog->CloseDialog();
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::TaskException(TObject* ASender, Exception* AException)
{
	if(AException->InheritsFrom(__classid(EAbort))) {
		FAsyncException = new EAbort(AException->Message);
	}
	else {
		FAsyncException = new ERenameError(AException->Message);
	}
}
//---------------------------------------------------------------------------
__fastcall TRenameCommand::TRenameCommand(TListViewModel* AControl) :
	TActionCommand(),
	FControl(AControl),
	FInvoker(NULL),
	FTask(NULL),
	FAsyncException(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TRenameCommand::~TRenameCommand()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Execute()
{
	Execute(new TDoRenameTask(Control, Invoker));
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Undo()
{
	Execute(new TUndoRenameTask(Control, Invoker));
}
//---------------------------------------------------------------------------
void __fastcall TRenameCommand::Redo()
{
	Execute(new TRedoRenameTask(Control, Invoker));
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TRenameSubCommand::Rename(bool AForward)
{
	if(OldValue == NewValue) {
		Item->Result = reSkipped;
		return;
	}
	AnsiString LPath = TPath::Compose(Item->Directory, AForward ? OldValue : NewValue);
	AnsiString LName = AForward ? NewValue : OldValue;
	if(!TFiles().TryRename(LPath, LName)) {
		Item->Result = reFailed;
		throw ERenameError("The program could not rename the file.");
	}
	Item->OldName = LName;
	Item->Result = reSucceeded;
}
//---------------------------------------------------------------------------
__fastcall TRenameSubCommand::TRenameSubCommand(TRenameItem* AItem) :
	TCommand(),
	FItem(AItem),
	FOldValue(Item->OldName),
	FNewValue(Item->NewName)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TRenameSubCommand::~TRenameSubCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TRenameSubCommand::Execute()
{
	Rename(true);
	Results = Results << crCompleted << crUndoable << crRedoable;
}
//---------------------------------------------------------------------------
void __fastcall TRenameSubCommand::Undo()
{
	Rename(false);
	Results = Results << crCompleted << crUndoable << crRedoable;
}
//---------------------------------------------------------------------------
void __fastcall TRenameSubCommand::Redo()
{
	Rename(true);
	Results = Results << crCompleted << crUndoable << crRedoable;
}
//---------------------------------------------------------------------------
} // End namespace Oren_renamecommand
//---------------------------------------------------------------------------

